/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.util.*;
import java.awt.*;

/**
 * Un oggetto appartenente alla classe <code>METAButton</code>  un pulsante etichettato
 * in grado di generare degli eventi ogni qualvolta all'interno della sua superficie avvengano
 * pressione o rilascio del mouse o, a prescindere dalla posizione del mouse, venga premuto un hotKey
 * associato al <code>METAButtom</code>.
 *
 * @see METAKeyPressedEvent
 * @see METAKeyReleasedEvent
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto
 */
public class METAButton extends METAComponent implements IButton
{
    /** Stringa di testo contenuta nel bottone. */
	private String label;

	/** Colore si sfondo del bottone. */
    private Color bgColor;

	/** Colore del testo contenuto nel bottone. */
    private Color textColor;

	/** Font del testo contenuto nel bottone. */
    private Font font;

	/** Carattere dell'hotKey associato al bottone. */
    private Character hotKey;

	/**
	 * Crea un nuovo bottone di tipo <code>METAButton</code> con le caratteristiche iniziali specificate dai parametri.
	 *
	 * @param width larghezza del bottone
	 * @param height altezza del bottone
	 * @param label stringa di testo contentura inizialmente nel bottone
	 * @param bg colore di sfondo del bottone
	 * @param text colore del testo contenuto nel bottone
	 * @param f font del testo
	 * @param hk carattere dell'hotkey associato al bottone
	 */
    public METAButton(int width, int height, String label, Color bg, Color text, Font f, char hk)
    {
	this.width = width;
	this.height = height;
	this.label = label;
	hotKey = new Character(hk);
	bgColor = bg;
	textColor = text;
	font = f;
    }
	
	/**
	 * Si occupa di disegnare il bottone nell'oggetto <code>Graphics</code> specificato.
	 *
	 * @param g oggetto graphics in cui disegnare il bottone.
	 */
    public void paint(Graphics g)
    {
	this.graphics = g;
	g.setColor(bgColor);
	g.draw3DRect(0, 0, this.width, this.height, true);
	g.fill3DRect(0, 0, this.width, this.height, true);
	g.setColor(textColor);
	g.setFont(font);
	FontMetrics fontM = g.getFontMetrics();
	g.drawString(label, (this.width - fontM.stringWidth(label)) / 2, this.height / 2 + fontM.getAscent() / 2);
    }

    /** Si occupa di ridisegnare il bottone durante la pressione del mouse o dell'hotkey associato. */
    private void paintRedraw()
    {
	this.graphics.setColor(bgColor);
	this.graphics.draw3DRect(0, 0, this.width, this.height, false);
	this.graphics.fill3DRect(0, 0, this.width, this.height, false);
	this.graphics.setColor(textColor);
	FontMetrics fontM = this.graphics.getFontMetrics();
	this.graphics.drawString(label, (this.width - fontM.stringWidth(label)) / 2, this.height / 2 + fontM.getAscent() / 2);
    }

    /**
	 * Si occupa di verificare se il componente  in grado gestire l'evento passato e restituisce true se l'evento  stato
	 * consumato, false in caso contrario.
	 */
    public boolean handle(IEvent i)
    {
	if(i.getClass().getName().equals("METAKeyPressedEvent")  &&  hotKey.compareTo(new Character(((METAKeyPressedEvent)i).getChar())) == 0)
	    paintRedraw();
	    
 	if(i.getClass().getName().equals("METAKeyReleasedEvent")  &&  hotKey.compareTo(new Character(((METAKeyReleasedEvent)i).getChar())) == 0)
	    {
		paint(this.graphics);
		if(this.getListener() != null) this.getListener().handle(this, i);
  		else return false;
	    }

 	if(i.getClass().getName().equals("METAMousePressedEvent"))
 	    paintRedraw();

 	if(i.getClass().getName().equals("METAMouseReleasedEvent"))
 	    {
  		paint(this.graphics);
       		if(this.getListener() != null) this.getListener().handle(this, new METAKeyReleasedEvent(label.charAt(0)));
  		else return false;
 	    }
	
	return true;
    }

	/** Restituisce una stringa contenente il testo del bottone. */
    public String getLabel()
    {
	return label;
    }
	
	/** Modifica la stringa di testo contenuta nel bottone. */
    public void setLabel(String label)
    {
	this.label = label;
    }
	
	/** Modifica il carattere rappresentante l'hotkey associato al bottone. */
    public void setHotKey(char ch)
    {
	hotKey = new Character(ch);
    }
	
	/** Restituisce il carattere rappresentante l'hotkey associato al bottone. */
    public char getHotKey()
    {
	return hotKey.charValue();
    }
}

