/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Un oggetto appartenente alla classe <code>METAWindow</code> rappresenta la finestra
 * principale dell'applicazione.
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto
 */
public class METAWindow extends JFrame implements IWindow 
{
    /** Contenuto della barra del titolo della finestra. */
	private final String TITLE;

	/**
	 * Container associato alla <code>METAWindow</code> con lo scopo di contenere
	 * tutti i componenti da visualizzare all'interno della finestra.
	 *
	 * @see METAContainer
	 */
    private METAContainer metaContainer;

	/** Contiene il riferimento all'istanza di JPanel che rappresenta lo spazio contenuto all'interno del frame. */
    private JPanel canv = new JPanel();
	
	/**
	 * Crea una nuova finestra di tipo <code>METAWindow</code> con le caratteristiche iniziali
	 * specificate dai parametri.
	 *
	 * @param title testo contenuto nella barra del titolo della finestra
	 * @param bg colore di sfondo della finestra
	 * @param d dimensioni della finestra
	 */
    public METAWindow(String title, Color bg, Dimension d)
    {
	TITLE = title;
	setTitle(TITLE);
	setSize(d.width, d.height);
	canv.setBackground(bg);
	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(canv, BorderLayout.CENTER);
	
	/** 
	 * aggiunge un listener sulla finestra che, nel caso in cui la finestra venga chiusa,
	 * causa la chiusura dell'intera applicazione.
	 */
	addWindowListener(new WindowAdapter() 
	    { 	
		public void windowClosing(WindowEvent e)
		{
		    System.exit(0);
		}
	    });
    }

    /**
	 * Si occupa di disegnare i componenti contentuti nell'oggetto <code>Graphics</code> specificato
	 * appartenente al container della finestra.
	 */
	public void paint(Graphics g)
    {
	super.paint(g);
	Graphics local_graphics = canv.getGraphics();
	metaContainer.paint(local_graphics);
    }
    
	/** Restituisce il titolo della finestra. */
    public String getTitle()
    {
	return TITLE;
    }
	
	/** Modifica il contenuto del container associato alla finestra <code>METAWindow</code>. */
    public void setContent(IComponent comp)
    {
	metaContainer = (METAContainer)comp;

	/** Aggiunge alla finestra un listener sulla tastiera che intercetta la pressione / il rilascio di un tasto. */
	addKeyListener(new KeyAdapter()
	    {
		
		/** 
		 * Nel caso in cui venga premuto un tasto della tastiera genera un evento <code>METAKeyPressedEvent</code>
		 * e lo passa al container che si occupera' di gestirlo passandolo a tutti i sottocomponenti.
		 */
		public void keyPressed(KeyEvent k)
		{
		    metaContainer.handle(new METAKeyPressedEvent(k.getKeyChar()));
		}
		
		/**
		 * Nel caso in cui venga rilasciato un tasto della tastiera genera un evento <code>METAKeyReleasedEvent</code>
		 * e lo passa al container della finestra che si occupera' di gestirlo e di passandolo a tutti i sottocomponenti.
		 */
		public void keyReleased(KeyEvent k)
		{
		    metaContainer.handle(new METAKeyReleasedEvent(k.getKeyChar()));
		}

	    });
	
	/** Aggiunge alla finestra un listener sul mouse che intercetta la pressione / il rilascio del pulsante sinistro del mouse. */
	canv.addMouseListener(new MouseAdapter() 
	    {

		/**
		 * Nel caso in cui venga premuto il pulsante sinistro del mouse genera un evento <code>METAMousePressedEvent</code>
		 * e lo passa al container che si occupera' di gestirlo passandolo a tutti i sottocomponenti.
		 */
		public void mousePressed(MouseEvent e) 
		{
		    metaContainer.handle(new METAMousePressedEvent(e.getPoint()));
		}	
		
		/**
		 * Nel caso in cui venga rilasciato il pulsante sinistro del mouse genera un evento <code>METAMouseReleasedEvent</code>
		 * e lo passa al container che si occupera' di gestirlo passandolo a tutti i sottocomponenti.
		 */
		public void mouseReleased(MouseEvent e) 
		{
		    metaContainer.handle(new METAMouseReleasedEvent(e.getPoint()));
		}	
	    });
    }
	
	/** Restituisce il container contenente i componenti della finestra. */
    public IComponent getContent()
    {
	return metaContainer;
    }
}

